# encoding: utf-8
"""
@project: djangoModel->region_api
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis:
@created_time: 2022/12/6 10:56
"""
import json
from time import time

from django.core import serializers
from django.http import JsonResponse
from django.views import View
from django.views.decorators.http import require_http_methods

from ..models import Region
from ..services.public_service import parse_data, model_del
from ..services.regin_service import RegionService
from ..validate import RegionValidate


# =============== 地区操作 =======================
class RegionList(View):
    # 查询列表
    @require_http_methods(["GET"])
    def get_child_region(self, code):
        # 获取下级的行政地区，0获取省级
        res_set = Region.objects.filter(p_code=code, is_delete=0)
        res_set = json.loads(serializers.serialize('json', res_set))
        final_res_dict = []
        for i in res_set:
            final_res_dict.append(i['fields'])
        return JsonResponse({'isSuccess': True, 'msg': '加载成功', 'data': final_res_dict})

    @require_http_methods(["GET"])
    def get_tree_region(self):
        start_time = time()
        service = RegionService()
        region_tree = service.tree_lock_loop()
        end_time = time()
        return JsonResponse({'run_time': end_time - start_time, 'isSuccess': True, 'msg': '加载成功', 'data': region_tree})


class RegionCreate(View):
    # 添加操作
    def post(self, request):
        data = parse_data(request.POST)
        validator = RegionValidate(data)
        is_valid, error = validator.validate()
        if not is_valid:
            return JsonResponse({'isSuccess': False, 'msg': error})
        res = Region.objects.filter(code=data['code'], is_delete=0)
        if res:
            return JsonResponse({'isSuccess': False, 'msg': '该数据已存在'})
        Region.objects.create(**data)
        return JsonResponse({'isSuccess': True, 'msg': '添加成功', 'data': ''})


class RegionUpdate(View):
    # 地区修改
    def post(self, request):
        id = request.POST.get('id')
        if not id:
            return JsonResponse({'isSuccess': False, 'msg': 'ID 必传'})
        data = parse_data(request.POST)
        res_obj = Region.objects.filter(id=id, is_delete=0)
        if not res_obj:
            return JsonResponse({'isSuccess': False, 'msg': '该数据不存在'})
        del data['id']
        res_obj.update(**data)
        return JsonResponse({'isSuccess': True, 'msg': '修改成功', 'data': ''})


class RegionDel(View):
    # 删除地区
    def post(self, request):
        res = model_del(request, Region, True)
        return JsonResponse(res)


class RegionCodeParse(View):

    def parse_code(self, code):
        # 解析code
        service = RegionService()
        res = service.cn_parse_location_code(code)
        return JsonResponse({'isSuccess': True, 'msg': '加载成功', 'data': res})



# encoding: utf-8
"""
@project: djangoModel->service
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: 公共模块服务 CURD 封装
@created_time: 2022/6/10 16:04
"""
import json

from django.core import serializers
from django.core.paginator import Paginator


def parse_data(data):
    # 解析请求参数
    requestData = {}
    for k, v in data.items():
        requestData[k] = v if not v == "" else None
    return requestData


def parse_model(res_set, need_all=False):
    json_data = json.loads(serializers.serialize('json', res_set))
    if not json_data:
        return None
    else:
        if not need_all:
            return json_data[0]['fields']
        else:
            return [i['fields'] for i in json_data]


def model_select(request, model, is_need_delete=False, json_parse_key=None):
    # 模型快速分页查询  分页+条件
    page = request.GET.get('page', 1)
    limit = request.GET.get('limit', 20)
    params = parse_data(request.GET)
    if 'page' in params.keys():
        del params['page']
    if 'limit' in params.keys():
        del params['limit']

    if is_need_delete:
        params['is_delete'] = 0
    try:
        list_set = model.objects.filter(**params)
        count = model.objects.filter(**params).count()
    except Exception as e:
        return {'isSuccess': False, 'msg': e.__str__(), 'data': None}
    # 分页数据
    limit_set = Paginator(list_set, limit)
    page_set = limit_set.get_page(page)
    # 数据序列化操作
    json_data = json.loads(serializers.serialize('json', page_set))
    final_res_dict = []
    for i in json_data:
        fields = i['fields']
        fields['id'] = i['pk']
        if not json_parse_key is None:
            fields[json_parse_key] = json.loads(fields[json_parse_key])
        final_res_dict.append(fields)
    # 数据拼装

    result = {'isSuccess': True, 'msg': '加载成功', 'data': final_res_dict}
    result['limit'] = int(limit)
    result['page'] = int(page)
    result['count'] = count
    return result


def model_del(request, model, is_real_delete=True):
    # 删除设备
    id = request.POST.get('id')
    if not id:
        return {'isSuccess': False, 'msg': 'ID不能为空'}
    from_data = parse_data(request.POST)
    if is_real_delete:
        res = model.objects.filter(**from_data)
        if not res:
            return {'isSuccess': False, 'msg': '数据已不存在'}
        res.delete()
    else:
        from_data['is_delete'] = 0
        res = model.objects.filter(**from_data)
        if not res:
            return {'isSuccess': False, 'msg': '数据已不存在'}
        res.update(is_delete=1)

    return {'isSuccess': True, 'msg': '删除成功'}


def model_create(request, model, validate):
    try:
        requestData = parse_data(request.POST)
        validator = validate(requestData)
        is_pass, error = validator.validate()
        if not is_pass:
            return {'isSuccess': False, 'msg': error}
        model.objects.create(**requestData)
    except Exception as e:
        return {'isSuccess': False, 'msg': str(e)}
    return {'isSuccess': True, 'msg': '创建成功'}


def model_update(request, model, is_need_delete=False):
    # 模型修改
    id = request.POST.get('id')
    if not id:
        return {'isSuccess': False, 'msg': 'ID不能为空'}
    from_data = parse_data(request.POST)

    del from_data['id']
    if from_data == {}:
        return {'isSuccess': False, 'msg': '修改项为空'}

    if is_need_delete:
        res = model.objects.filter(id=id, is_delete=0)
    else:
        res = model.objects.filter(id=id)
    if not res:
        return {'isSuccess': False, 'msg': '数据已不存在'}
    try:
        res.update(**from_data)
        return {'isSuccess': True, 'msg': '修改成功'}
    except Exception as e:
        return {'isSuccess': False, 'msg': str(e)}



# encoding: utf-8
"""
@project: djangoModel->regin_service
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis:
@created_time: 2022/6/10 16:08
"""
import json
import time
import uuid

from django_redis import get_redis_connection

from ..models import Region
from ..services.public_service import parse_model


class RegionService():
    def cn_parse_location_code(self, region_code):
        # 中国行政编码解析 province{2},city(2),district(2),town(3),village(3)三位
        # level？ province ==>>village : 1 ===>> 5
        if not region_code or not len(str(region_code)) == 12:
            return None
        location = {}
        region_code = str(region_code)
        location['province'] = self.get_region_info(region_code[0:2] + "0000000000")

        city = self.get_region_info(region_code[0:4] + "00000000")
        location['city'] = city if not city == location['province'] else None

        district = self.get_region_info(region_code[0:6] + "000000")
        location['district'] = district if not district == city else None

        town = self.get_region_info(region_code[0:9] + "000")
        location['town'] = town if not town == district else None

        village = self.get_region_info(region_code[0:])
        location['village'] = village if not town == village else None

        return location

    def get_region_info(self, code):
        # 查询对应的名称
        json_data = parse_model(Region.objects.filter(code=code, is_delete=0))
        if not json_data:
            return None
        else:
            return json_data

    def tree_loop(self, p_code=0, level=1):
        # 树形数据遍历
        first_level = parse_model(Region.objects.filter(level=level, is_delete=0, p_code=p_code), True)
        if first_level:
            for j in first_level:
                j['child'] = self.tree_loop(j['code'], j['level'] + 1)
        return first_level

    def tree_lock_loop(self):
        # redis 分布式锁 TODO 守护进程防止锁超时和死锁
        conn = get_redis_connection()
        region_tree = conn.get('region_tree')
        if not region_tree:  # 不存在缓存的时候
            # 当缓存失效的时候需要使用redis 锁进行阻塞，防止雪崩现象
            lock = conn.get('region_tree_lock')
            if not lock:  # 未加锁时候
                lock_key = uuid.getnode()
                conn.set('region_tree_lock', lock_key)
                conn.expire('region_tree_lock', 3)
                region_tree = self.tree_loop()
                conn.set('region_tree', json.dumps(region_tree))
                conn.delete('region_tree_lock')
            else:  # 已经被枷锁，进行自旋
                times = 0
                while True:
                    region_tree = conn.get('region_tree')
                    if not region_tree is None:
                        region_tree = json.loads(region_tree)
                        break
                    if times > 6:
                        lock = conn.get('region_tree_lock')
                        if not lock:
                            self.tree_lock_loop()
                        return None
                    times += 1
                    time.sleep(0.5)
        else:  # 存在缓存之恶返回
            region_tree = json.loads(region_tree)
        return region_tree



from django.db import models

# from apps.user.models import User

SHARD_TABLE_NUM = 3


class Region(models.Model):
    id = models.IntegerField(primary_key=True, null=False, auto_created=True)
    code = models.IntegerField('行政编码', null=False, default=0)
    p_code = models.IntegerField('上级行政编码', null=False, default=0)
    name = models.CharField('地区名称', null=False, default="", max_length=50)
    level = models.IntegerField('行政划分等级', null=False, default="0")
    is_delete = models.IntegerField('是否删除', null=False, default=0)

    class Meta:
        managed = False
        db_table = "append_public_region"
        verbose_name = "行政区编码表"
        verbose_name_plural = verbose_name



from django.urls import re_path

from xj_common.api.translate_apis import TranslateApis

urlpatterns = [
    # 公共翻译
    re_path(r'^translate_article/?$', TranslateApis.translate_article, ),
    re_path(r'^translate_test/?$', TranslateApis.as_view(), ),
    # 公共定位服务
]
